RFRMT-SEXPR 1.0 BASE64

(table "rformat-version-major"
       1
       "rformat-version-minor"
       0
       "subformat"
       (table "identifier" 	"reng_resource"
              "name" 	"REng resource"
              "description" 	"REng engine asset")
       "root-node"
       (array (array "result"
                     "inline"
                     (array "shader"
                            "fragment"
                            "#version 330 core

precision highp float;

layout(std140) uniform transData
{
    mat4 posTrans;
    mat3 uvTrans;
};

layout(std140) uniform worldInfo
{
    vec3 camPos;
    mat3 uvToView;
    vec4 exposure;
    vec4 deferredIndex;
};

uniform sampler2D baseColorTex;
uniform sampler2D metalTex;
uniform sampler2D roughTex;
uniform sampler2D normalTex;
uniform sampler2D depthTex;
uniform sampler2D lightTex;
uniform sampler2D positionTex;
uniform sampler2D ssaoTex;
uniform sampler2D envTex;
uniform sampler2D shadowTex;

varying vec2 uv;

vec3 colorOut;
out vec4 FragColor;
const float PI = 3.1415926535897932;


float linearizeDepth(float d)
{
    const float zNear = .001;
    const float zFar = 1000;

    d = 2.0 * d - 1.0;
    float l = 2.0 * zNear * zFar / (zFar + zNear - d * (zFar - zNear));
    return l;
}


vec4 textureEqui(sampler2D sampler, vec3 coord)
{
    vec2 coord2d = vec2(atan(coord.x, coord.z),
                        acos(-coord.y));
    coord2d /= vec2(2*PI, PI);

    return texture2D(sampler, coord2d);
}


float distribution_GGX(vec3 N, vec3 H, float alpha)
{
    float alpha2 = alpha * alpha;
    float NoH    = dot(N, H);
    float NoH2   = NoH*NoH;

    float num    = alpha2;
    float denom  = (NoH2 * (alpha2 - 1.0) + 1.0);
    denom        = PI * denom * denom;

    return num / denom;
}


float geometry_SchlickGGX(float NoD, float rough)
{
    float r = (rough + 1.0);
    float k = (r*r) / 8.0;

    float num   = NoD;
    float denom = NoD * (1.0 - k) + k;

    return num / denom;
}


float geometry_Smith(float NoV, float NoL, float rough)
{
    float ggxV = geometry_SchlickGGX(NoV, rough);
    float ggxL = geometry_SchlickGGX(NoL, rough);

    return ggxV * ggxL;
}

vec3 fresnel_SchlickRoughness(float cosTheta, vec3 F0, float rough)
{
    return F0 + (max(vec3(1.0 - rough), F0) - F0) * pow(1.0 - cosTheta, 5.0);
}  


vec3 visSign(float v)
{
    if (v > 0)
      return vec3(0, 0, min(1, v));

    if (v < 0)
      return vec3(min(1, -v), 0, 0);

    return vec3(0, 0, 0);
}

vec3 pbrShade(vec3 N, vec3 V, vec3 L,
	      vec3 baseColor, float rough,
	      vec3 kD, vec3 kS,
	      vec3 radiance)
{
    vec3  H     = normalize(V + L);
    float NoV   = dot(N, V);
    float NoL   = dot(N, L);
    float NoV0  = max(NoV, 0);
    float NoL0  = max(NoL, 0);

    float alpha = rough * rough;

    // diffuse
    vec3 diff   = kD * baseColor / PI;

    // specular
    float NDF   = distribution_GGX(N, H, alpha);
    float G     = geometry_Smith(NoV0, NoL0, rough);

    vec3  num   = NDF * G * kS;
    float denom = 4 * NoV * NoL;
    vec3  spec  = num / denom;

    vec3 result = (diff + spec) * radiance * NoL0;
    return result;
}

void main ()
{
    vec3  baseColor  = texture2D(baseColorTex, uv).rgb;
    float metal      = texture2D(metalTex, uv).r;
    float rough      = texture2D(roughTex, uv).r;
    vec3  N          = normalize(texture2D(normalTex, uv).rgb);
    float depth      = linearizeDepth(texture2D(depthTex, uv).r);
	float shadowMap  = linearizeDepth(texture2D(shadowTex, uv).r);
    vec3  light      = texture2D(lightTex, uv).rgb;
    vec3  worldPos   = texture2D(positionTex, uv).rgb;
    float ssao       = texture2D(ssaoTex, uv).r;
    //float ssao = 1;

    vec3  V          = normalize(camPos - worldPos);

    // temporary light
    vec3 lightValue  = vec3(1,1,.6) * 2;
    vec3 lightDir    = vec3(-1, 1, -1);

    vec3 F0 = mix(vec3(0.04), baseColor, metal);
    vec3 kS = fresnel_SchlickRoughness(max(dot(N, V), 0.0), F0, rough);
    vec3 kD = vec3(1.0) - kS;
    kD *= 1.0 - metal;

    vec3 Lo = light;

    // for each light ...
    vec3  L        = lightDir;
    vec3  radiance = lightValue;

    Lo += pbrShade(N, V, L,
  		 baseColor, rough,
  		 kD, kS,
  		 radiance);

    // apply color
    vec3 ambient = vec3(0.001, 0.001, 0.001) * baseColor;
    vec3 colorOut = ambient + Lo;
	colorOut = light;
    colorOut *= ssao;

    // sky
    vec3 skyCoord = normalize(uvToView * vec3(uv*2-1, 1));
    skyCoord.y = -skyCoord.y;
    vec3 skyColor = textureEqui(envTex, skyCoord).rgb;

    colorOut = depth>10 ? skyColor : colorOut;

    // tonemap
    colorOut *= exposure.x;
    colorOut = colorOut / (colorOut+1);

    //colorOut = vec3(ssao);
    //colorOut = vec3(length(N));

    colorOut = (deferredIndex.x<.1) ? colorOut : N;

    // gamma correct
    colorOut = pow(colorOut, vec3(1/2.2));
	FragColor = vec4(colorOut,1);
	//FragColor = vec4(vec3(linearizeDepth(texture2D(shadowTex, uv).r))/10,1);
	//FragColor = vec4(colorOut,1) + vec4(vec3(linearizeDepth(texture2D(shadowTex, uv).r))/10,0.3f);
}  


"))))
