RFRMT-SEXPR 1.0 BASE64

(table "rformat-version-major"
       1
       "rformat-version-minor"
       0
       "subformat"
       (table "identifier" 	"reng_resource"
              "name" 	"REng resource"
              "description" 	"REng engine asset")
       "root-node"
       (array (array "result"
                     "inline"
                     (array "shader"
                            "fragment"
                            "#version 330 core

layout(std140) uniform lightData
{
	mat4 lightMatrix;
	vec3 position;
	vec3 color;
	vec3 direction;
};

uniform sampler2D baseColorTex;
uniform sampler2D normalTex;
uniform sampler2D positionTex;
uniform sampler2D shadowTex;

in vec2 uv;
in vec4 smCoords;

out vec3 FragColor;

float linearizeDepth(float d)
{
    const float zNear = .001;
    const float zFar = 10000;

    d = 2.0 * d - 1.0;
    float l = 2.0 * zNear * zFar / (zFar + zNear - d * (zFar - zNear));
    return l;
}

float calcShadows()
{  
	// transform to light space
	vec3 worldPos = texture2D(positionTex, uv).rgb;
    vec4 fragProj = lightMatrix * vec4(worldPos, 1);
    vec3 fragLightUv = fragProj.xyz / fragProj.w;
    fragLightUv = fragLightUv * .5 + .5;
	
	// get closest depth
	float nearestDist = texture(shadowTex,  fragLightUv.xy).r;
    nearestDist = linearizeDepth(nearestDist);
	
	// get current depth
    float realDist = distance(worldPos, position);
	//float realDist = fragLightUv.z;
	
	// bias
    vec3 normal = normalize(texture2D(normalTex, uv).rgb);
    vec3 lightDir = direction;
    float bias = max(0.05 * (1.0 - dot(normal, lightDir)), 0.005);
	
	//return realDist > nearestDist  ? 1.0 : 0.0; 
	
	// PCF
    float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowTex, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowTex, fragLightUv.xy + vec2(x, y) * texelSize).r; 
			pcfDepth = linearizeDepth(pcfDepth);
            shadow += realDist > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 15.0;
	
	return shadow;
}

void main()
{   
	float constant = 1.0f;
    float linear = 0.7f;
    float quadratic = 1.8f;
    vec3  baseColor  = texture2D(baseColorTex, uv).rgb;
	vec3  worldPos   = texture2D(positionTex, uv).rgb;
    vec3  N          = normalize(texture2D(normalTex, uv).rgb);
	
	float cutO = cos(radians(25.f));
	float cutOO= cos(radians(30.f));	
	
	//diffuse
	vec3 dir = normalize(position - worldPos);
	float diff = max(dot(N, dir), 0.0);
	vec3 diffuse = diff * color * baseColor;
    
	
	// spotlight (soft edges)
    float theta = dot(dir, normalize(-direction)); 
    float epsilon = (cutO - cutOO);
    float intensity = clamp((theta - cutOO) / epsilon, 0.0, 1.0);
    
	diffuse  *= intensity;
	
	// attenuation
	float distance    = length(position - worldPos);
	float attenuation = 1.0 / (constant + linear * distance + quadratic * (distance * distance));    
	
	diffuse  *= attenuation;
	
	float shadow = calcShadows();
    
	vec3 lighting = (1.0-shadow)*diffuse;    

    FragColor = lighting;
}  "))))
