#version 330


layout(std140) uniform shaderData
{
  mat4  transform;

  float shear;

  // borders
  float b1;
  float b2;
  float b3;

  // colors
  vec4 c1;
  vec4 c2_1;
  vec4 c2_2;
  vec4 c3;
};

uniform sampler2D distTex;

in vec2 uv;
out vec4 fragColor;

void main ()
{
  float v = texture(distTex, uv).r;
  float f = length(fwidth(v)) * .6f;

  // masks
  float m1 = clamp((v-b1+f)/(2*f) , 0, 1);
  float m2 = clamp((v-b2+f)/(2*f) , 0, 1);
  float m3 = clamp((v-b3+f)/(2*f) , 0, 1);

  float m12 = clamp(m1 - m2, 0, 1);
  float m23 = clamp(m2 - m3, 0, 1);

  // color weights
  float c2_1w = m12 * (v-b1) / (b2-b1);
  float c1w   = m12 * (1 - c2_1w);
  
  float c3w   = m23 * (v-b2) / (b3-b2);
  float c2_2w = m23 * (1 - c3w);
  
  // combine the colors
  fragColor = vec4(c1.rgb,   1) * c1.w * c1w + vec4(c2_1.rgb, 1) * c2_1.w * c2_1w + vec4(c2_2.rgb, 1) * c2_2.w * c2_2w + vec4(c3.rgb,   1) * c3.w   * c3w;
}

